﻿package com.selenetan.core 
{
	import com.selenetan.core.STObject;
    import com.selenetan.aspturtle.MapSection;
	import flash.display.Sprite;
	
	/**
     * A STObject that holds other objects inside of it and updates them together.
     * @author Selene
     */
    public class STContainer extends STObject
    {
        
        public var children:Array;
        
        protected var _lastX:Number;
        protected var _lastY:Number;
        protected var _lastAngle:Number;
        
        public function STContainer($sprite:Sprite, $hitSprite:Sprite = null, $x:Number = 0, $y:Number = 0) 
        {
            super($sprite, $hitSprite, $x, $y);
			
            children = [];
            
            _lastX = x;
            _lastY = y;
            _lastAngle = angle;
        }
        
        /**
         * Add an object as a child of this container. 
         * Its sprite will be added as a child of this container's sprite.
         * Children will have their motion properties updated when the STContainer's
         * properties are updated.
         * @param	object
         */
        public function addAsChild(object:STObject):void
        {
            children.push(object);
            sprite.addChild(object.sprite);
        }
        
        
        override public function update():void
        {
            super.update();
            updateChildren();
            /*
            updateMotion();
            //updateChildren();
            
            _lastX = x;
            _lastY = y;
            _lastAngle = angle;
            */
        }
        
        protected function updateChildren():void
        {
            var numChildren:int = children.length;
            for (var i:int = 0; i < numChildren; ++i) {
                var child:STObject = STObject(children[i]);
                child.update();
            }
            
            /*****
             * ? Since the children sprites are children of this sprite, their x/y/angle should
             * be propertly updated already.
            if (_lastX == x && _lastY == y && _lastAngle == angle) {
                return;
            }
            
            var deltaX:Number = x - _lastX;
            var deltaY:Number = y - _deltaY;
            var deltaAngle:Number = angle - _lastAngle;
            
            var numChildren:int = children.length;
            for (var i = 0; i < numChildren; ++i) {
                var child:STObject = STObject(children[i]);
                child.sprite.x += deltaX;
                child.sprite.y += deltaY;
                child.rotation -= deltaAngle;
                
            }
            *****/
        }
        
        override public function render():void
        {
            var numChildren:int = children.length;
            for (var i:int = 0; i < numChildren; ++i) {
                var child:STObject = STObject(children[i]);
                child.render();
            }
            
            super.render();
        }
        
    }

}