﻿package com.selenetan.core 
{
    import flash.utils.getTimer;
	/**
     * ...
     * @author Selene
     */
    public class STUtil
    {
        /**
         * Check if any of the objects from group1 are colliding with any objects from group2.
         * @param	group1      The first collision group
         * @param	group2      The second collision group
         * @param	callback    This function should take two STObjects as arguments.
         */
        public static function checkCollisions(group1:Array, group2:Array, callback:Function=null):void
        {
            var group1length:int = group1.length;
            var group2length:int = group2.length;
            
            for (var i:int = 0; i < group1length; ++i) {
                var obj1:STObject = STObject(group1[i]);
                
                if (obj1.dead) {
                    continue;
                }
                
                var j:int;
                if (group1 != group2) {
                    j = 0;
                }
                else {
                    j = i + 1;
                }
                for ( ; j < group2length; ++j) {
                    var obj2:STObject = STObject(group2[j]);
                    
                    if (obj2.dead || obj1 == obj2) {
                        continue;
                    }
                    
                    // **TODO?: Optimize this check
                    if (obj1.hitSprite.hitTestObject(obj2.hitSprite)) {
                        var deltaX:Number = obj1.hitSprite.x  - obj2.hitSprite.x;
                        var deltaY:Number = obj1.hitSprite.y  - obj2.hitSprite.y;
                        var distSquared:Number = deltaX * deltaX + deltaY * deltaY;
                        var totalRadiusSquared:Number = (obj1.hitRadius + obj2.hitRadius) * (obj1.hitRadius + obj2.hitRadius);
                        
                        if (distSquared < totalRadiusSquared) {
                            //trace(obj1.hitSprite.name + " collided with " + obj2.hitSprite.name + "!");
                            //trace("  distSquared: " + distSquared);
                            //trace("  totalRadiusSquared: " + totalRadiusSquared);
                            
                            if (callback != null) {
                                callback(obj1, obj2);
                            }
                        }
                    }
                }
            }
        }
        
        /**
         * Check if any of the objects from group1 are colliding with any objects from group2,
         * and update the objects' velocities in response.
         * @param	group1      The first collision group.
         * @param	group2      The second collision group.
         * @param	callback    This function should take two STObjects as arguments.
         */
        public static function checkResolveCollisions(group1:Array, group2:Array, callback:Function = null):void
        {
            var group1length:int = group1.length;
            var group2length:int = group2.length;
            
            var currWorld:STWorld = STGlobal.currentWorld;
            
            for (var i:int = 0; i < group1length; ++i) {
                var obj1:STObject = STObject(group1[i]);
                if (obj1.dead) {
                    continue;
                }
                
                var j:int;
                if (group1 != group2) {
                    j = 0;
                }
                else {
                    j = i + 1;
                }
                for (; j < group2length; ++j) {
                    var obj2:STObject = STObject(group2[j]);
                    
                    if (obj2.dead || obj1 == obj2) {
                        continue;
                    }
                    
                    
                    var deltaX:Number;
                    var deltaY:Number;
                    
                    if (currWorld.wrapping & STWorld.WRAPPING_HORIZONTAL) {
                        deltaX = (obj1.x  - obj2.x + currWorld.worldWidth * 0.5) % currWorld.worldWidth - currWorld.worldWidth * 0.5;
                    }
                    else {
                        deltaX = obj1.x  - obj2.x;
                    }
                    if (currWorld.wrapping & STWorld.WRAPPING_VERTICAL) {
                        deltaY = (obj1.y - obj2.y + currWorld.worldHeight * 0.5) % currWorld.worldHeight - currWorld.worldHeight * 0.5;
                    }
                    else {
                        deltaY = obj1.y  - obj2.y;
                    }
                    var distSquared:Number = deltaX * deltaX + deltaY * deltaY;
                    var totalRadiusSquared:Number = (obj1.hitRadius + obj2.hitRadius) * (obj1.hitRadius + obj2.hitRadius);
                    
                    if (distSquared < totalRadiusSquared) {
                        //trace(obj1.sprite.name + " collided with " + obj2.sprite.name + " at " + getTimer() + "ms");
                        //trace("  distSquared: " + distSquared + ", totalRadiusSquared: " + totalRadiusSquared);
                        //trace("  obj1.radius: " + obj1.hitRadius + ", obj2.radius: " + obj2.hitRadius);
                        //trace("  obj1.x,y: " + obj1.x + "," + obj1.y + "    obj2.x,y: " + obj2.x + "," + obj2.y);
                        //trace("  obj1.vel: " + obj1.velocity + ", obj2.vel: " + obj2.velocity);
                        
                        var dist:Number = Math.sqrt(distSquared);
                        var overlapAmount:Number = Math.sqrt(totalRadiusSquared) - dist;
                        
                        /*
                        // Try to separate objects so they're not colliding
                        var massTotal:Number = obj1.mass + obj2.mass
                        var commonMultiplier:Number = overlapAmount / (dist * massTotal);
                        obj1.x += deltaX * obj2.mass * commonMultiplier;
                        obj2.x -= deltaX * obj1.mass * commonMultiplier;
                        obj1.y += deltaY * obj2.mass * commonMultiplier;
                        obj2.y -= deltaY * obj1.mass * commonMultiplier;
                        */
                        /*
                        // Back the objects out of the collision so they don't keep bouncing in each other
                        var elapsed:Number = STGlobal.ticker.elapsed;
                        trace("  elapsed: " + elapsed);
                        obj1.x -= elapsed * obj1.velocity.x;
                        obj1.y -= elapsed * obj1.velocity.y;
                        obj2.x -= elapsed * obj2.velocity.x;
                        obj2.y -= elapsed * obj2.velocity.y;
                        */
                        
                        
                        // Get the positions and velocities in the reference frame of Object 1
                        var vel2Xin1:Number = obj2.velocity.x - obj1.velocity.x;
                        var vel2Yin1:Number = obj2.velocity.y - obj1.velocity.y;
                        
                        var acc2Xin1:Number = obj2.acceleration.x - obj1.acceleration.x;
                        var acc2Yin1:Number = obj2.acceleration.y - obj1.acceleration.y;
                        
                        var x2in1:Number = -deltaX;
                        var y2in1:Number = -deltaY;
                        
                        var totalRadius:Number = obj1.hitRadius + obj2.hitRadius;
                        
                        var a:Number = vel2Xin1 * vel2Xin1 + vel2Yin1 * vel2Yin1;
                        var b:Number = -2 * (x2in1 * vel2Xin1 + y2in1 * vel2Yin1);
                        var c:Number = x2in1 * x2in1 + y2in1 * y2in1 - totalRadius * totalRadius;
                        
                        var determinant:Number = b * b - 4 * a * c;
                        if (determinant < 0) {
                            trace ("waaaah, negative determinant: " + determinant);
                        }
                        else {
                            determinant = Math.sqrt(determinant);
                        }
                        
                        
                        var timeSinceCollision:Number;
                        if (( -b - determinant) > 0) {
                            timeSinceCollision = ( -b - determinant) / (2 * a);
                        }
                        else {
                            timeSinceCollision = ( -b + determinant) / (2 * a);
                        }
                        
                        var elapsed:Number = STGlobal.ticker.elapsed;
                        //trace("  Elapsed: " + elapsed);
                        //trace("  Time since collision: " + timeSinceCollision);
                        
                        obj1.x -= timeSinceCollision * obj1.velocity.x;
                        obj1.y -= timeSinceCollision * obj1.velocity.y;
                        obj2.x -= timeSinceCollision * obj2.velocity.x;
                        obj2.y -= timeSinceCollision * obj2.velocity.y;
                        
                        
                        //trace("  After backing out:\n  obj1.x,y: " + obj1.x + "," + obj1.y + "    obj2.x,y: " + obj2.x + "," + obj2.y);
                        
                        resolveXCollision(obj1, obj2);
                        resolveYCollision(obj1, obj2);
                        
                        //trace("  After collision:\n  obj1.vel: " + obj1.velocity + ", obj2.vel: " + obj2.velocity);
                        
                        
                        obj1.x += timeSinceCollision * obj1.velocity.x;
                        obj1.y += timeSinceCollision * obj1.velocity.y;
                        obj2.x += timeSinceCollision * obj2.velocity.x;
                        obj2.y += timeSinceCollision * obj2.velocity.y;
                        
                        if (callback != null) {
                            callback(obj1, obj2);
                        }
                    }
                }
            }
        }
        
        /**
         * Resolves a collision on the X axis between two STObjects.
         * This function directly updates the velocities of the two objects.
         * @param	object1
         * @param	object2
         */
        public static function resolveXCollision(object1:STObject, object2:STObject):void
        {
            var momentum1:Number = object1.mass * object1.velocity.x;
            var momentum2:Number = object2.mass * object2.velocity.x;
            
            var totalMass:Number = object1.mass + object2.mass;
            var velDiff:Number = object2.velocity.x - object1.velocity.x;
            
            var elasticity:Number = object1.elasticity * object2.elasticity;
            
            var vFinal1:Number = (elasticity * object2.mass * velDiff + momentum1 + momentum2) / totalMass;
            var vFinal2:Number = (elasticity * object1.mass * -velDiff + momentum1 + momentum2) / totalMass;
            
            object1.velocity.x = vFinal1;
            object2.velocity.x = vFinal2;
            
            //trace("  " + object1.sprite.name + ".xvel: " + object1.velocity.x);
            //trace("  " + object2.sprite.name + ".xvel: " + object2.velocity.x);
        }
        
        
        /**
         * Resolves a collision on the Y axis between two STObjects.
         * This function directly updates the velocities of the two objects.
         * @param	object1
         * @param	object2
         */
        public static function resolveYCollision(object1:STObject, object2:STObject):void
        {
            var momentum1:Number = object1.mass * object1.velocity.y;
            var momentum2:Number = object2.mass * object2.velocity.y;
            
            var totalMass:Number = object1.mass + object2.mass;
            var velDiff:Number = object2.velocity.y - object1.velocity.y;
            
            var elasticity:Number = object1.elasticity * object2.elasticity;
            
            var vFinal1:Number = (elasticity * object2.mass * velDiff + momentum1 + momentum2) / totalMass;
            var vFinal2:Number = (elasticity * object1.mass * -velDiff + momentum1 + momentum2) / totalMass;
            
            object1.velocity.y = vFinal1;
            object2.velocity.y = vFinal2;
        }
        
        /**
         * Clamps the given value to either +max or -max depending on sign
         * @param	value   The value to clamp
         * @param	max     The maximum possible value
         * @return  The value, +max, or -max.
         */
        public static function clamp(value:Number, max:Number):Number
        {
            if (value > max) {
                value = max;
            }
            else if (value < -max) {
                value = -max;
            }
            
            return value;
        }
        
    }

}